%global with_check 0
%define gobuild(o:) \
GO111MODULE=off go build -buildmode pie -compiler gc -tags="rpm_crashtraceback ${BUILDTAGS:-}" -ldflags "${LDFLAGS:-} -linkmode=external -compressdwarf=false -B 0x$(head -c20 /dev/urandom|od -An -tx1|tr -d ' \\n') -extldflags '%__global_ldflags'" -a -v %{?**};

%global import_path github.com/containers/podman
%global cataver 0.1.7
%global dnsnamever  1.3.0
%global gvproxyver 0.6.1

Summary:       Manage Pods, Containers and Container Images
Name:          podman
Version:       4.5.1
Release:       3%{?dist}
License:       ASL 2.0 and GPLv3+
URL:           https://%{name}.io/
Source0:       https://%{import_path}/archive/refs/tags/v%{version}.tar.gz #/%{name}-%{version}.tar.gz
Source1:       https://github.com/openSUSE/catatonit/archive/v%{cataver}.tar.gz #/catatonit-%{cataver}.tar.gz
Source2:       https://github.com/containers/dnsname/archive/v%{dnsnamever}.tar.gz #/dnsname-%{dnsnamever}.tar.gz
Source3:       https://github.com/containers/gvisor-tap-vsock/archive/v%{gvproxyver}.tar.gz #/gvisor-tap-vsock-%{gvproxyver}.tar.gz

BuildRequires: gcc make autoconf automake pkgconfig libtool
BuildRequires: glib2-devel glibc-devel glibc-static git-core
BuildRequires: gpgme-devel libassuan-devel libgpg-error-devel
BuildRequires: libseccomp-devel libselinux-devel ostree-devel
BuildRequires: systemd systemd-devel shadow-utils-subid-devel
BuildRequires: golang /usr/bin/go-md2man file gettext
Requires:      containers-common fuse-overlayfs oci-runtime
Requires:      netavark slirp4netns conmon >= 2.0.25
Requires:      container-selinux iptables nftables libseccomp >= 2.5
Suggests:      containernetworking-plugins
Recommends:    crun
Provides:      %{name}-manpages = %{version}-%{release}
Provides:      %{name}-catatonit = %{version}-%{release}

%description
%{name} (Pod Manager) is a fully featured container engine that is a simple daemonless tool.
%{name} provides a Docker-CLI comparable command line that eases the transition from
other container engines and allows the management of pods, containers and images.
Simply put: alias docker=%{name}.
Most %{name} commands can be run as a regular user, without requiring
additional privileges.

%{name} uses Buildah(1) internally to create container images.
Both tools share image (not container) storage, hence each can use or
manipulate images (but not containers) created by the other.


%package docker
Summary:       Emulate Docker CLI using %{name}
BuildArch:     noarch
Requires:      %{name} = %{version}-%{release}
Conflicts:     docker docker-ce moby

%description docker
This package installs a script named docker that emulates the Docker CLI by
executes %{name} commands, it also creates links between all Docker CLI man
pages and %{name}.


%package remote
Summary: A remote CLI for Podman: A Simple management tool for pods, containers and images

%description remote
%{name}-remote provides a local client interacting with a Podman backend node through
a RESTful API tunneled through a ssh connection. In this context, a %{name} node is
a Linux system with Podman installed on it and the API service activated.

Credentials for this session can be passed in using flags, environment variables,
or in containers.conf.


%package plugins
Summary:       Plugins for %{name}
Requires:      dnsmasq
Recommends:    %{name}-gvproxy = %{version}-%{release}

%description plugins
This plugin sets up the use of dnsmasq on a given CNI network so that Pods can
resolve each other by name.  When configured, the pod and its IP address are
added to a network specific hosts file that dnsmasq will read in.
Similarly, when a pod is removed from the network, it will remove the entry from
the hosts file.  Each CNI network will have its own dnsmasq instance.


%package tests
Summary:       Tests for %{name}
Requires:      %{name} = %{version}-%{release}
Requires:      nmap-ncat httpd-tools jq socat
Requires:      skopeo buildah
Requires:      openssl gnupg git-daemon

%description tests
This package contains system tests for %{name}


%package gvproxy
Summary: Go replacement for libslirp and VPNKit

%description gvproxy
A replacement for libslirp and VPNKit, written in pure Go.
It is based on the network stack of gVisor. Compared to libslirp,
gvisor-tap-vsock brings a configurable DNS server and dynamic port forwarding.


%prep
%autosetup -Sgit -n %{name}-%{version}
sed -i 's;@@PODMAN@@\;$(BINDIR);@@PODMAN@@\;%{_bindir};' Makefile
sed -i 's,-Werror,,' pkg/rootless/rootless_linux.go
tar fx %{SOURCE1}
pushd catatonit-%{cataver}
sed -i '$d' configure.ac
popd
tar fx %{SOURCE2}
tar fx %{SOURCE3}

rm -rf docs/source/markdown/containers-mounts.conf.5.md


%build
pushd catatonit-%{cataver}
autoreconf -fi
%configure
CFLAGS="%{optflags} -fPIE -D_GNU_SOURCE -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -D_FILE_OFFSET_BITS=64"
%{make_build}

file catatonit | grep 'statically linked'
if [ $? != 0 ]; then
   echo "ERROR: catatonit binary must be statically linked!"
   exit 1
fi
popd

export GO111MODULE=off
export GOPATH=$(pwd)/_build:$(pwd)
CGO_CFLAGS="%{optflags} -D_GNU_SOURCE -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE -D_FILE_OFFSET_BITS=64"
CGO_CFLAGS=$(echo $CGO_CFLAGS | sed 's/-flto=auto//g')
CGO_CFLAGS=$(echo $CGO_CFLAGS | sed 's/-Wp,D_GLIBCXX_ASSERTIONS//g')

%ifarch x86_64
export CGO_CFLAGS+=" -m64 -mtune=generic -fcf-protection=full"
%endif

mkdir _build
pushd _build
mkdir -p src/github.com/containers
ln -s ../../../../ src/github.com/containers/podman
popd
ln -s vendor src

rm -rf vendor/github.com/containers/storage/drivers/register/register_btrfs.go

unset LDFLAGS
LDFLAGS="-X %{import_path}/libpod/define.buildInfo=$(date +%s)"

%gobuild -o bin/rootlessport %{import_path}/cmd/rootlessport

export BUILDTAGS="seccomp btrfs_noversion exclude_graphdriver_devicemapper exclude_graphdriver_btrfs \
                  $(hack/libdm_tag.sh) $(hack/selinux_tag.sh) $(hack/systemd_tag.sh) $(hack/libsubid_tag.sh)"
%gobuild -o bin/%{name} %{import_path}/cmd/%{name}

export BUILDTAGS="remote $BUILDTAGS"
%gobuild -o bin/%{name}-remote %{import_path}/cmd/%{name}

%gobuild -o bin/quadlet %{import_path}/cmd/quadlet

%{__make} docs


unset LDFLAGS
pushd dnsname-%{dnsnamever}
mkdir _build
pushd _build
mkdir -p src/github.com/containers
ln -s ../../../../ src/github.com/containers/dnsname
popd
ln -s vendor src
export GOPATH=$(pwd)/_build:$(pwd)
%gobuild -o bin/dnsname github.com/containers/dnsname/plugins/meta/dnsname
popd


pushd gvisor-tap-vsock-%{gvproxyver}
mkdir _build
pushd _build
mkdir -p src/github.com/containers
ln -s ../../../../ src/github.com/containers/gvisor-tap-vsock
popd
ln -s vendor src
export GOPATH=$(pwd)/_build:$(pwd)
%gobuild -o bin/gvproxy github.com/containers/gvisor-tap-vsock/cmd/gvproxy
popd


%install
PODMAN_VERSION=%{version} %{__make} PREFIX=%{buildroot}%{_prefix} ETCDIR=%{buildroot}%{_sysconfdir} \
        install.bin \
        install.remote \
        install.man \
        install.systemd \
        install.completions \
        install.docker

sed -i 's;%{buildroot};;g' %{buildroot}%{_bindir}/docker
ln -s ./ ./vendor/src
install -d -p %{buildroot}/%{_datadir}/%{name}/test/system
cp -pav test/system %{buildroot}/%{_datadir}/%{name}/test/
rm -f               %{buildroot}/%{_datadir}/%{name}/test/system/*.t

for file in `find %{buildroot}%{_mandir}/man[15] -type f | sed "s,%{buildroot},," | grep -v -e remote -e docker`; do
    echo "$file*" >> podman.file-list
done

install -dp %{buildroot}%{_libexecdir}/podman
install -dp %{buildroot}%{_datadir}/licenses/podman
install -p catatonit-%{cataver}/catatonit %{buildroot}%{_libexecdir}/podman/catatonit
install -p catatonit-%{cataver}/COPYING %{buildroot}%{_datadir}/licenses/podman/COPYING-catatonit

pushd dnsname-%{dnsnamever}
%{__make} PREFIX=%{_prefix} DESTDIR=%{buildroot} install
popd

pushd gvisor-tap-vsock-%{gvproxyver}
install -dp %{buildroot}%{_libexecdir}/%{name}
install -p -m0755 bin/gvproxy %{buildroot}%{_libexecdir}/%{name}
popd


%check
%if 0%{?with_check}
ln -s ./ ./vendor/src

export GOPATH=%{buildroot}/%{gopath}:$(pwd)/vendor:%{gopath}

%if ! 0%{?gotest:1}
%global gotest go test
%endif

%gotest %{import_path}/cmd/%{name}
%gotest %{import_path}/libkpod
%gotest %{import_path}/libpod
%gotest %{import_path}/pkg/registrar
%endif


%preun
if [ $1 == 0 ]; then
  systemctl stop podman.service > /dev/null 2>&1
  systemctl stop podman.socket > /dev/null 2>&1
  systemctl disable podman.service > /dev/null 2>&1
  systemctl disable podman.socket > /dev/null 2>&1
fi
:


%files -f podman.file-list
%license LICENSE COPYING-catatonit
%doc README.md CONTRIBUTING.md install.md transfer.md
%{_bindir}/%{name}
%dir %{_libexecdir}/%{name}
%{_libexecdir}/%{name}/catatonit
%{_libexecdir}/%{name}/quadlet
%{_libexecdir}/%{name}/rootlessport
%ghost %dir %{_sysconfdir}/cni/net.d
%ghost %{_sysconfdir}/cni/net.d/87-%{name}-bridge.conflist
%{_unitdir}/*.service
%{_unitdir}/*.socket
%{_unitdir}/*.timer
%{_userunitdir}/*.service
%{_userunitdir}/*.socket
%{_userunitdir}/*.timer
%{_usr}/lib/tmpfiles.d/%{name}.conf
%{_usr}/lib/systemd/system-generators/podman-system-generator
%{_usr}/lib/systemd/user-generators/podman-user-generator
%{_datadir}/bash-completion/completions/%{name}
%dir %{_datadir}/zsh/site-functions
%{_datadir}/zsh/site-functions/_%{name}
%dir %{_datadir}/fish/vendor_completions.d
%{_datadir}/fish/vendor_completions.d/%{name}.fish


%files docker
%{_bindir}/docker
%{_usr}/lib/tmpfiles.d/%{name}-docker.conf
%{_datadir}/user-tmpfiles.d/podman-docker.conf


%files remote
%license LICENSE
%{_bindir}/%{name}-remote
%{_mandir}/man1/%{name}-remote*.*
%{_datadir}/bash-completion/completions/%{name}-remote
%dir %{_datadir}/fish
%dir %{_datadir}/fish/vendor_completions.d
%{_datadir}/fish/vendor_completions.d/%{name}-remote.fish
%dir %{_datadir}/zsh
%dir %{_datadir}/zsh/site-functions
%{_datadir}/zsh/site-functions/_%{name}-remote


%files plugins
%license dnsname-%{dnsnamever}/LICENSE
%doc dnsname-%{dnsnamever}/{README.md,README_PODMAN.md}
%{_libexecdir}/cni/dnsname


%files tests
%license LICENSE
%{_datadir}/%{name}/test


%files gvproxy
%license gvisor-tap-vsock-%{gvproxyver}/LICENSE
%doc gvisor-tap-vsock-%{gvproxyver}/README.md
%dir %{_libexecdir}/%{name}
%{_libexecdir}/%{name}/gvproxy


%changelog
* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 4.5.1-3
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Aug 04 2023 Xiaojie Chen <jackxjchen@tencent.com> - 4.5.1-2
- Rebuilt for file 5.45

* Wed Jul 12 2023 kianli <kianli@tencent.com> - 4.5.1-1
- Upgrade to 4.5.1

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 4.4.4-4
- Rebuilt for OpenCloudOS Stream 23.05

* Mon Apr 17 2023 rockerzhu <rockerzhu@tencent.com> - 4.4.4-3
- Delete annobin treatment in flags.

* Fri Apr 07 2023 rockerzhu <rockerzhu@tencent.com> - 4.4.4-2
- Don't provides docker, being provided by moby.

* Fri Apr 07 2023 rockerzhu <rockerzhu@tencent.com> - 4.4.4-1
- Initial build.
